unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, CheckLst,
  Laz2_XMLCfg;

type

  { TForm1 }

  TForm1 = class(TForm)
    bAdd1: TButton;
    bAdd2: TButton;
    bDelete1: TButton;
    bDelete2: TButton;
    bLoad: TButton;
    bSave: TButton;
    cbDNS1: TCheckListBox;
    cbDNS2: TCheckListBox;
    lbDNS1: TLabel;
    lbDNS2: TLabel;
    procedure bAdd1Click(Sender: TObject);
    procedure bDelete1Click(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
  private
    procedure SaveToXML(const APath: String);
    procedure LoadFromXML(const APath: String);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

type
  TDest = record
    FNumber: String;
    FChecked: Boolean;
  end;

  TDNS = record
    FName: String;
    FInclude: Integer;
    FDest: array of TDest;
  end;

var
  DNS: array of TDNS;


procedure TForm1.SaveToXML(const APath: String);
var
  FXML: TXMLConfig;
  Path, SubPath: String;
  DNSCnt: Integer;
  DestCnt: Integer;
  I, J: Integer;
begin
  FXML := TXMLConfig.CreateClean(APath);
  try
    DNSCnt := Length(DNS);
    FXML.SetDeleteValue('DNSCount/Value', DNSCnt, 0);
    for I := 0 to DNSCnt - 1 do
    begin
      Path := 'DNS' + IntToStr(I) + '/';
      DestCnt := Length(DNS[I].FDest);
      FXML.SetDeleteValue(Path + 'Name', DNS[I].FName, '');
      FXML.SetDeleteValue(Path + 'Include', DNS[I].FInclude, 0);
      FXML.SetDeleteValue(Path + 'DestCnt', DestCnt, 0);
      for J := 0 to DestCnt - 1 do
      begin
        SubPath := Path + 'Destination' +  IntToStr(J) + '/';
        FXML.SetDeleteValue(SubPath + 'Number', DNS[I].FDest[J].FNumber, '');
        FXML.SetDeleteValue(SubPath + 'Checked', DNS[I].FDest[J].FChecked, False);
      end;
    end;
    FXML.Flush;
  finally
    FXML.Free;
  end;
end;

procedure TForm1.LoadFromXML(const APath: String);
var
  FXML: TXMLConfig;
  Path, SubPath: String;
  DNSCnt: Integer;
  DestCnt: Integer;
  I, J: Integer;
begin
  if not FileExists(APath) then
    Exit;
  FXML := TXMLConfig.Create(APath);
  try
    DNSCnt := FXML.GetValue('DNSCount/Value', 0);
    SetLength(DNS, DNSCnt);
    for I := 0 to DNSCnt - 1 do
    begin
      Path := 'DNS' + IntToStr(I) + '/';
      DNS[I].FName := FXML.GetValue(Path + 'Name', '');
      DNS[I].FInclude := FXML.GetValue(Path + 'Include', 0);
      DestCnt := FXML.GetValue(Path + 'DestCnt', 0);
      SetLength(DNS[I].FDest, DestCnt);
      for J := 0 to DestCnt - 1 do
      begin
        SubPath := Path + 'Destination' +  IntToStr(J) + '/';
        DNS[I].FDest[J].FNumber := FXML.GetValue(SubPath + 'Number', '');
        DNS[I].FDest[J].FChecked := FXML.GetValue(SubPath + 'Checked', False);
      end;
    end;
  finally
    FXML.Free;
  end;
end;

procedure TForm1.bAdd1Click(Sender: TObject);
var
  Number: String;
begin
  Number := InputBox('New number', 'Enter a number:', '');
  if Number = '' then
    Exit;
  case TButton(Sender).Tag of
    0: cbDNS1.Items.Add(Number);
    1: cbDNS2.Items.Add(Number);
  end;
end;

procedure TForm1.bDelete1Click(Sender: TObject);
var
  cb: TCheckListBox;
begin
  case TButton(Sender).Tag of
    0: cb := cbDNS1;
    1: cb := cbDNS2;
  end;
  cb.DeleteSelected;
end;

procedure TForm1.bLoadClick(Sender: TObject);
var
  I, J: Integer;
begin
  LoadFromXML(ExtractFilePath(Application.Exename) + 'config.xml');
  cbDNS1.Clear;
  cbDNS2.Clear;
  for I := Low(DNS) to High(DNS) do
  begin
    for J := Low(DNS[I].FDest) to High(DNS[I].FDEst) do
      case I of
         0: begin
              lbDNS1.Caption := DNS[I].FName + '(Include: ' + IntToStr(DNS[I].FInclude) + ')';
              cbDNS1.Items.Add(DNS[I].FDest[J].FNumber);
              cbDNS1.Checked[J] := DNS[I].FDest[J].FChecked;
         end;
         1: begin
              lbDNS2.Caption := DNS[I].FName + '(Include: ' + IntToStr(DNS[I].FInclude) + ')';
              cbDNS2.Items.Add(DNS[I].FDest[J].FNumber);
              cbDNS2.Checked[J] := DNS[I].FDest[J].FChecked;
         end;
       end;
  end;
end;

procedure TForm1.bSaveClick(Sender: TObject);
var
  I: Integer;
begin
  DNS := nil;
  SetLength(DNS, 2); //only 2 dns in this case

  DNS[0].FName := 'DNS1';
  DNS[0].FInclude := 12;
  SetLength(DNS[0].FDest, cbDNS1.Count);
  for I := 0 to cbDNS1.Count - 1 do
  begin
    DNS[0].FDest[I].FNumber := cbDNS1.Items.Strings[I];
    DNS[0].FDest[I].FChecked := cbDNS1.Checked[I];
  end;

  DNS[1].FName := 'DNS2';
  DNS[1].FInclude := 34;
  SetLength(DNS[1].FDest, cbDNS2.Count);
  for I := 0 to cbDNS2.Count - 1 do
  begin
    DNS[1].FDest[I].FNumber := cbDNS2.Items.Strings[I];
    DNS[1].FDest[I].FChecked := cbDNS2.Checked[I];
  end;
  SaveToXML(ExtractFilePath(Application.Exename) + 'config.xml');
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  bSaveClick(bSave);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  bLoadClick(bLoad);
end;


end.

